#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import pandas as pd

from setup import Setup as setup


class AssPar:
    """Storage for parameters needed for the assessment."""
    SELECTION_CRITERION = 'AIC'
    BINARY_ORDER = 3
    UN_RES_HEADER = ['MFlowT', 'MQlowT', 'BP', 'MFhighT', 'MQhighT', 'DiffEl', 'Matrix', 'Mode', 'DP']
    BIN_RES_HEADER = ['HighestOrder', 'DiffEl', 'IntEl1', 'IntEl2', 'Order', 'MQlowT', 'BP', 'MQhighT', 'AIC', 'BIC',
                      'Mode', 'DP']
    TERN_RES_HEADER = ['Mode', 'DiffEl', 'IntEl1', 'IntEl2', 'IntEl3', 'MainEl', 'MQlowT', 'BP', 'MQhighT', 'AIC',
                       'BIC', 'DP']
    data = pd.DataFrame()
    temp_data = pd.DataFrame()
    od = True
    alloy = setup.ELEMENTS
    phases = setup.PHASES
    _un_res = pd.DataFrame()
    _bin_res = pd.DataFrame()
    _tern_res = pd.DataFrame()

    @staticmethod
    def set_results():
        """Initializes the dataframes to store the unary, binary and ternary results"""
        AssPar.un_res = pd.DataFrame(columns=AssPar.UN_RES_HEADER)
        AssPar.bin_res = pd.DataFrame(columns=AssPar.BIN_RES_HEADER)
        AssPar.tern_res = pd.DataFrame(columns=AssPar.TERN_RES_HEADER)

    @staticmethod
    def add_unary_result(res:pd.DataFrame):
        if AssPar._un_res.empty:
            AssPar._un_res = res
        else:
            AssPar._un_res = AssPar._un_res.append(res)
        AssPar._un_res.reset_index(drop=True, inplace=True)

    @staticmethod
    def get_unary_results() ->pd.DataFrame:
        return AssPar._un_res.copy()

    @staticmethod
    def add_binary_result(res:pd.DataFrame):
        if AssPar._bin_res.empty:
            AssPar._bin_res = res
        else:
            AssPar._bin_res = AssPar._bin_res.append(res)
        AssPar._bin_res.reset_index(drop=True, inplace=True)

    @staticmethod
    def get_binary_results() ->pd.DataFrame:
        return AssPar._bin_res.copy()

    @staticmethod
    def add_ternary_result(res:pd.DataFrame):
        if AssPar._tern_res.empty:
            AssPar._tern_res = res
        else:
            AssPar._tern_res = AssPar._tern_res.append(res)
        AssPar._tern_res.reset_index(drop=True, inplace=True)

    @staticmethod
    def get_ternary_results() ->pd.DataFrame:
        return AssPar._tern_res.copy()

