#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import numpy as np


class ErrorManager:

    __slots__ = ('_num_par', '_data', '_pred_dat', 'results')

    def __init__(self, num_par: int, data, pred_dat):
        self._num_par = num_par
        self._data = data
        self._pred_dat = pred_dat
        self.results = dict()

    def get_errors(self):
        """Calculates all possible errors"""
        self.get_AIC()
        self.get_BIC()

    def get_RSS(self):
        """RSS: Residual sum of squares. Variation of data around estimated regression line"""
        rss = 0.0
        for i in range(len(self._data)):
            rss = rss + (self._data[i]-self._pred_dat[i])**2
        if rss < 0.0:
            print('Error in RSS calculation!')
            exit(1)
        self.results['RSS'] = rss
        #new function:
    def get_RSE(self):
        """RSE: Residual error RSS/(n-p-1)"""
        rse = 0.0
        for i in range(len(self._data)):
            rse = rse + (self._data[i]-self._pred_dat[i])**2
        if rse < 0.0:
            print('Error in RSE calculation!')
            exit(1)
        if len(self._data)-self._num_par-1>0:
            self.results['RSE'] = np.sqrt(rse/(len(self._data)-self._num_par-1))
        else:
            self.results['RSE'] = np.sqrt(rse)

    def get_AIC(self):
        """AIC_ Akaike Information Criterion"""
        self.get_RSS()
        if (len(self._data)-self._num_par-1)!=0:
            self.results['AIC'] = 2*self._num_par + len(self._data)*np.log(self.results['RSS']/len(self._data)) + (2*self._num_par *self._num_par +2*self._num_par)/(len(self._data)-self._num_par-1)
        else:
            self.results['AIC'] = 2*self._num_par + len(self._data)*np.log(self.results['RSS']/len(self._data)) + (2*self._num_par *self._num_par +2*self._num_par)

    def get_BIC(self):
        """BIC: Bayesian Information Criteria"""
        self.get_RSS()
        try:
            if self.results['RSS'] <= 0:
                raise RuntimeWarning
        except RuntimeWarning:
            print("In log:", len(self._data) * self._num_par * self.results['MSE'])
            print("number data: ", len(self._data))
            print("RSS: ", self.results['RSS'])
            print("Number parameters: ", self._num_par)
            print("MSE: ", self.results['MSE'])
            print("Nenner: ", len(self._data) * (self.results['RSS'] + np.log(len(self._data) * self._num_par * self.results['MSE'])))
            print("Error in BIC calculation")
            exit(7)
        else:
            self.results['BIC'] = len(self._data) *np.log(self.results['RSS']/len(self._data))+ np.log(len(self._data)) * self._num_par
