#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import pandas as pd

from general import InitRes as rm
from add_data import add_data as ad
from assessment import main as ass
from db import initialize as init
from output import ddb
from output import rawdata_anal as rda
from setup import ReadData as rd
from setup import Setup as set


class InputReader:

    @classmethod
    def main(cls):
        """Initialize SQL databases if they do not exist and reads main input"""
        if set.DEBUG:
            pd.set_option('mode.chained_assignment', 'raise')
        init.InitDB().main()
        if rd.ADD_DATA:
            ad.DataReader().main()
        all_phases = set.PHASES
        rm.init_output(all_phases)
        rda.AnalyseRD().analyse()
        ass.Assessment().main()
        ddb.DDBWriter().main()


if __name__ == '__main__':
    InputReader.main()

