#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import matplotlib.pyplot as plt
import numpy as np

import general as gf
from output import parameter as pp


class UnPl:
    """Handles all unary plots."""

    def __init__(self, diff_el, matrix, phase, data, assres):
        self._el = diff_el
        self._matrix = matrix
        self._phase = phase
        self._data = data
        self._assres = assres
        self._path = gf.General.PATH

    def main(self):
        self._init_output()
        self._xy_plot("Temp", "D", sort_refs=True)
        self._xy_plot("invT", "lnD", sort_refs=True)
        self._xy_plot("Temp", "D", sort_refs=False)
        self._xy_plot("invT", "lnD", sort_refs=False)

    def _init_output(self):
        self._path = self._path + "/Results"
        gf.General.create_dir(self._path)
        self._path = self._path + "/Endmember_Results"
        gf.General.create_dir(self._path)
        self._path = self._path + "/" + self._phase
        gf.General.create_dir(self._path)
        self._path = self._path + "/" + self._el + "_in_" + self._matrix
        gf.General.create_dir(self._path)

    def _xy_plot(self, x_var, y_var, sort_refs=True):
        gc = pp.PlotParameter.gas_const
        factor = 1
        if x_var == "invT":
            factor = pp.PlotParameter.invT_fact
        if self._data is not None:
            refs = gf.DataManipulator.get_list_of_references(self._data)
            #print(self._data)
            col = pp.PlotParameter.get_colour(0)
            for i in range(0, len(refs)):
                counter = 0
                for j in range(0, len(self._data)):
                    label = '_nolegend_'
                    if sort_refs:
                        col = pp.PlotParameter.get_colour(i)
                    if self._data.loc[j, 'Original'] == refs.iloc[i, 0]:
                        if counter == 0:
                            try:
                                label = self._data.loc[j, 'Original']+ ", w: "+ str("{:.3f}".format(self._data.loc[j,'weight']))
                            except KeyError:
                                label = self._data.loc[j, 'Original']+ ", w: "+ str("{:.3f}".format(1))									  
                        if not sort_refs:
                            label = '_nolegend_'
                        plt.plot(self._data.loc[j, x_var] * factor, self._data.loc[j, y_var], 'x', color=pp.PlotParameter.get_colour(i), marker=pp.PlotParameter().get_marker(i),
                                 label=label)
                        counter += 1

        if not self._assres.empty:
            counter = 1
            bp = self._assres.loc[0, 'BP']
            if bp is None:
                mf = self._assres.loc[0, 'MFlowT']
                mq = self._assres.loc[0, 'MQlowT']
                lab = '_nolegend_'
                if y_var == "D":
                    x_range = pp.PlotParameter.get_xrange_temp(self._data.Temp.min(), self._data.Temp.max())
                    plt.plot(x_range, mf * np.exp(-mq /(gc * x_range)), color=pp.PlotParameter.get_colour(counter),
                                 label=lab)
                elif y_var == "M":
                    x_range = pp.PlotParameter.get_xrange_temp(self._data.Temp.min(), self._data.Temp.max())
                    pref = gc * x_range
                    plt.plot(x_range, 1/pref*mf *np.exp(- mq / (gc * x_range)), color=pp.PlotParameter.get_colour(counter),
                                 label=lab)
                elif y_var == "lnD":
                    x_range = pp.PlotParameter.get_xrange_itemp(self._data.invT.min(), self._data.invT.max()) * factor
                    plt.plot(x_range, np.log(mf) - mq / gc * x_range / factor,
                                 color=pp.PlotParameter.get_colour(counter), label=lab)
                elif y_var == "lnM":
                    x_range = pp.PlotParameter.get_xrange_itemp(self._data.invT.min(), self._data.invT.max()) * factor
                    plt.plot(x_range, np.log(mf/gc*x_range/factor)- mq / gc * x_range / factor, color=pp.PlotParameter.get_colour(counter),
                                 label=lab)
                counter = counter + 1
            else:
                mf_low_t = self._assres.loc[0, 'MFlowT']
                mq_low_t = self._assres.loc[0, 'MQlowT']
                mf_high_t = self._assres.loc[0, 'MFhighT']
                mq_high_t = self._assres.loc[0, 'MQhighT']
                lab = '_nolegend_'
                if y_var == "D":
                    x_range = pp.PlotParameter.get_xrange_temp(self._data.Temp.min(), self._data.Temp.max())
                    y_dat = []
                    for m in range(len(x_range)):
                        if x_range[m] <= bp:
                            y_dat.append(mf_low_t * np.exp(-mq_low_t /(gc * x_range[m])))
                        else:
                            y_dat.append(mf_high_t * np.exp(-mq_high_t /(gc * x_range[m])))
                    plt.plot(x_range, y_dat, color=pp.PlotParameter.get_colour(counter), label=lab)
                elif y_var == "M":
                    x_range = pp.PlotParameter.get_xrange_temp(self._data.Temp.min(), self._data.Temp.max())
                    pref = gc * x_range
                    y_dat = []
                    for m in range(len(x_range)):
                        if x_range[m] <= bp:
                            y_dat.append(1/pref[m]*mf_low_t * np.exp(-mq_low_t /(gc * x_range[m])))
                        else:
                            y_dat.append(1/pref[m]*mf_high_t * np.exp(-mq_high_t /(gc * x_range[m])))
                    plt.plot(x_range, y_dat, color=pp.PlotParameter.get_colour(counter), label=lab)

                elif y_var == "lnD":
                    x_range = pp.PlotParameter.get_xrange_itemp(self._data.invT.min(), self._data.invT.max()) * factor
                    y_dat = []
                    for m in range(len(x_range)):
                        if 1*factor/x_range[m] <= bp:
                            y_dat.append(np.log(mf_low_t) - mq_low_t / gc * x_range[m] / factor)
                        else:
                            y_dat.append(np.log(mf_high_t) - mq_high_t / gc * x_range[m] / factor)
                    plt.plot(x_range, y_dat, color=pp.PlotParameter.get_colour(counter), label=lab)

                elif y_var == "lnM":
                    x_range = pp.PlotParameter.get_xrange_itemp(self._data.invT.min(), self._data.invT.max()) * factor
                    y_dat = []
                    for m in range(len(x_range)):
                        if 1*factor/x_range[m] <= bp:
                            y_dat.append(np.log(mf_low_t/gc*x_range[m]/factor)- mq_low_t / gc * x_range[m] / factor)
                        else:
                            y_dat.append(np.log(mf_high_t/gc*x_range[m]/factor)- mq_high_t / gc * x_range[m] / factor)
                    plt.plot(x_range, y_dat, color=pp.PlotParameter.get_colour(counter), label=lab)

        handles, labels = plt.gca().get_legend_handles_labels()
        if labels:
            leg = plt.legend(loc='best', prop={'size': 6})
            leg.get_frame().set_alpha(0.5)
        xlab = "T [K]"
        if x_var == "invT":
            xlab = str(factor) + '/T [1/K]'
        ylab = 'D[m²/s]'
        if y_var == 'M':
            ylab = 'M'
        elif y_var == 'lnD':
            ylab = 'ln(D [m²/s])'
        elif y_var == 'lnM':
            ylab = 'ln(M)'
        plt.xlabel(xlab)
        plt.ylabel(ylab)
        tit = 'Diffusion Data of ' + self._el + " in " + self._matrix
        plt.title(tit)
        figname = self._path + '/' + y_var + "_vs_" + x_var
        if sort_refs:
            figname = figname + '_sorted_by_refs'
        plt.savefig(figname + pp.PlotParameter.fpdf, dpi=pp.PlotParameter.res, bbox_inches='tight')
        plt.savefig(figname + pp.PlotParameter.fpng, dpi=pp.PlotParameter.res, bbox_inches='tight')
        plt.close()







