#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import numpy as np

from setup import Setup as set


class PlotParameter:
    num_col = 39  # number of different colours available for the plots
    fpdf = ".pdf"
    fpng = ".png"
    gas_const = 8.314
    res = 900
    if set.DEBUG:
        res = 100
    shift_lnD = 1
    shift_D = 1E-13
    invT_fact = 10000
    markersize = 3

    @classmethod
    def get_colour(cls, i):
        """Returns a colour for the plot."""
        sorted_names = ['blue', 'red', 'green', 'black', 'olive', 'orange', 'indigo',  'teal', 'grey',
                        'cyan', 'brown',
                        'darkblue', 'purple', 'darkgreen', 'fuchsia', 'gold',
                        'aqua', 'coral', 'chocolate', 'navy', 'orangered', 'pink', 'plum', 'beige', 'magenta',
                        'maroon',
                        'chartreuse', 'crimson', 'aquamarine', 'azure',
                        'goldenrod', 'salmon', 'lightgreen', 'lightblue', 'orchid', 'lavender', 'black', 'ivory',
                        'khaki']
        return sorted_names[i % len(sorted_names)]

    @classmethod
    def get_marker(cls, i):
        """Returns a marker."""
        #marker = [".", "x", "X", "v", "^", "*", "d", "D", "s", "o", "<", ">", "1", "8", "p", "P","h", "H", "2", "+" "3", "4"]
        marker = ["o", "s", "X", "v", "^", "*", "d", "D", "<", ">", "1", "8", "p", "h", "H", "2", "+", "3", "4", "s", "o"]
        return marker[i % len(marker)]

    @classmethod
    def get_xrange_temp(cls, tmin=None, tmax=None):
        """Returns x-range for minimum and maximum temperature."""
        min_temp = 273
        max_temp = 3000
        diff = tmax - tmin
        if tmin is not None and not np.isnan(tmin):
            min_temp = tmin
            if diff < 50:
                min_temp = min_temp - 100
        if tmax is not None and not np.isnan(tmax):
            max_temp = tmax
            if diff < 50:
                max_temp = max_temp + 100
        return np.arange(min_temp, max_temp, (max_temp - min_temp) * 0.0001)

    @classmethod
    def get_xrange_itemp(cls, itmin=None, itmax=None):
        """Returns x-range for minimum and maximum temperature."""
        min_itemp = 0.0003
        max_itemp = 0.0037
        diff = 1/itmin - 1/itmax
        if itmin is not None and not np.isnan(itmin):
            min_itemp = itmin
            if diff < 50:
                min_itemp = 1/(1/min_itemp + 100)
        if itmax is not None and not np.isnan(itmax):
            max_itemp = itmax
            if diff < 50:
                max_itemp = 1/(1/max_itemp - 100)
        return np.arange(min_itemp, max_itemp, (max_itemp - min_itemp) * 0.0001)
