#
#   This file is part of the PyMob software project.
#   For more details visit www.icams.de
#
#   Created: 2020
#
#   Copyright (c) 2020-2021 Interdisciplinary Centre for Advanced Materials
#                 Simulation (ICAMS). Ruhr-Universitaet Bochum. Germany
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


class Setup:
    """This class contains parameters for the assessment.

    DEBUG: if true, gives additional output

    ELEMENTS: defines elements considered in the assessment; provided in a list

    PHASES: defines phases considered in the assessment; provided in a list

    """
    DEBUG = True
    #ELEMENTS = ["CR", "FE", "NI"]
    ELEMENTS = ["CO", "CR", "FE", "MN", "NI"]
    TOTAL_EL = ['AC', 'AG', 'AL', 'AR', "ARS", 'AT', 'AU', 'B', 'BA', 'BE', 'BH', 'BI', 'C', 'CA', 'CD', 'CE', 'CN',
                'CO', 'CR', 'CS', 'CU', 'DB', 'DS', 'ER', 'EU', 'F', 'FE', 'FR', 'GA', 'GD', 'GE', 'H', 'HE', 'HF',
                'HG', 'HO', 'HS', "IND", 'IR', 'K', 'KR', 'LA', 'LI', 'LU', 'N', 'NA', 'MG', 'MN', 'MO', 'MT', 'NB',
                'ND', 'NE', 'NI', 'O', 'OS', 'P', 'PB', 'PD', 'PM', 'PO', 'PT', 'PU', 'RA', 'RB', 'RE', 'RF', 'RG',
                'RH', 'RN', 'RU', 'S', 'SB', 'SC', 'SE', 'SG', 'SI', 'SM', 'SN', 'SR', 'TA', 'TC', 'TE', 'TH', 'TI',
                'TL', 'TM', 'U', 'V', 'W', 'XE', 'Y', 'YB', 'ZN', 'ZR']
    PHASES = ["FCC"]


class ReadData:
    """Contains information (path, filenames for raw data that should be read in. These paramerers are only used if
    'ADD_DATA=True.'

    A file containing the references should be given on first position, otherwise problems occur because the software
    does not accept data without the given reference in the database.

    The values must be given in SI-Units.

    Examples for such file can be found in the RawData directory.

    DATA_PATH: Path to the directory with raw data
    FILENAMES: List of one or more filenames that contain the data and that are located in the DATA_PATH directory
    AMOUNT_DATA: List of number of data in each file to check if all data are read in
    """
    ADD_DATA = True
    DATA_PATH = "./RawData"
    FILENAMES = ["Reference_File.csv", "RawData_File.csv"]
    AMOUNT_DATA = [2166, 3033]


class AssessmentSettings:
    """Assessment Settings can be manipulated here.

    SELECTION_CRITERION: Selection Criterion used for teh selection of binary and ternary interaction parameters.

    BINARY_ORDER: order for binary interactions in Redlich-Kister term. The models are tested up to this order.

    """


